

import java.awt.Color;
import java.awt.Dimension;
import java.awt.Font;
import java.awt.Graphics2D;
import java.awt.Image;
import java.awt.Toolkit;
import java.awt.image.BufferedImage;
import java.io.File;

import javax.swing.ImageIcon;
import javax.swing.UIManager;
import javax.swing.UnsupportedLookAndFeelException;

import TreeSnatcher.Core.MainWindow;
import TreeSnatcher.GUI.AboutBox;
import TreeSnatcher.GUI.FancyCursors;
import TreeSnatcher.Utils.ImageLoader;

public class TreeSnatcher
{
	private String path = null;
	private BufferedImage srcImage;
	private static TreeSnatcher instance = new TreeSnatcher();
	
	private TreeSnatcher()
	{
		System.setProperty("apple.laf.useScreenMenuBar", "true");
		System.setProperty("com.apple.mrj.application.apple.menu.about.name", "TreeSnatcher Plus");
		
		
		try 
		{
			//UIManager.setLookAndFeel(UIManager.getSystemLookAndFeelClassName());
			UIManager.setLookAndFeel(UIManager.getCrossPlatformLookAndFeelClassName());
		} 
		catch (ClassNotFoundException e){} 
		catch (InstantiationException e) {} 
		catch (IllegalAccessException e) {} 
		catch (UnsupportedLookAndFeelException e) {};
		
	}
	
	public static TreeSnatcher getInstance() 
	{
		return instance;
	}
	
	private void processArguments(String[] arguments)
	{
		// If the class is executed from command line, a path to an image file should be provided as parameter
		// If executed by double-clicking its icon, the class displays an empty image
		if (arguments.length > 0) 
		{
		    String currentDir = System.getProperty("user.dir");
		    this.path = currentDir+File.separator+"TreeSnatcher"+File.separator;
		    File inputFile = new File(path+arguments[0]);
		    inputFile = inputFile.getAbsoluteFile();
		    if (inputFile.exists()) 
		    {
		    	srcImage = ImageLoader.getImage(path+arguments[0]);
		    } 
		    else 
		    {
		    	inputFile = null;
		    	// File or directory does not exist
		        System.err.println("\nArgument must be a path to an image file, or the file does not exist");
		        System.exit(1);
		    }
		}
	}
	
	private void showMainWindow()
	{
		MainWindow mainView = new MainWindow(srcImage, path);
		mainView.setVisible(true);
	}

		
	public static void main(String[] args) 
	{   
		// Get a singleton of the TreeSnatcher class
		final TreeSnatcher ts = TreeSnatcher.getInstance();
		
		if (ts != null)
		{
			// Process the command line argument(s), if any
			ts.processArguments(args);
			// Let the application's main window take over control
			javax.swing.SwingUtilities.invokeLater(new Runnable() 
			{
				/*
				public class  HeapSize 
				{
				public static void main(String[]args)
				{
				long Size = Runtime.getRuntime().totalMemory();
				System.out.println("Heap Size = " + Size);
				}
				}
				*/

	            public void run() 
	            {
	            	Font font = new Font("SansSerif", Font.PLAIN, 12);
	            	Font buttonFont = new Font("SansSerif", Font.PLAIN, 11);
	            	UIManager.put("Button.margin", "2, 12, 2, 12");
	            	UIManager.put("Button.font", buttonFont);
	            	UIManager.put("ToggleButton.margin", "2, 12, 2, 12");
	            	UIManager.put("ToggleButton.font", buttonFont);
	            	UIManager.put("RadioButton.font", font);
	            	UIManager.put("Label.font", font);
	            	
	                ts.showMainWindow();
	            }
	        });
		}
	}

	
}
