/**
 * 
 */
package TreeSnatcher.Utils;

import java.text.DecimalFormatSymbols;
import java.text.NumberFormat;
import java.util.Iterator;
import java.util.Locale;
import java.util.Vector;

import TreeSnatcher.Core.Branch;

public class NumberUtility 
{
	private static NumberFormat numberFormat;
	
	public NumberUtility()
	{
		numberFormat = NumberFormat.getInstance(Locale.US);
		numberFormat.setMaximumFractionDigits(0);
		numberFormat.setMinimumFractionDigits(0);
		numberFormat.setGroupingUsed(false);
	}
	
	public static int getNumFractionDigits(double number)
	{
		String s = ((Double) number).toString();
		if (Math.rint(number) == number) return 0;
		else return s.substring(s.lastIndexOf(DecimalFormatSymbols.getInstance().getDecimalSeparator())+1).length();
	}
	
	public static void setNumFractionDigits(double number)
	{
		int n = getNumFractionDigits(number);
		setNumFractionDigits(n);
	}
	
	// Get the maximum number of fractional digits needed to represent all branch lengths
	// in Collection br
	// Set it as the current number of fractional digits for all numbers
	public static void setNumFractionDigits(Vector<Branch> br)
	{
		Iterator<Branch> it = br.iterator();
		int maxNumFractionDigits = Integer.MIN_VALUE;
		
		while (it.hasNext())
		{
			Branch b = it.next();
			double l = b.getLength();
			
			int numFractionDigits = getNumFractionDigits(l);
			if (numFractionDigits > maxNumFractionDigits)
			{
				maxNumFractionDigits = numFractionDigits;
			}
		}
		setNumFractionDigits(maxNumFractionDigits);
	}
	
	public static void setNumFractionDigits(int n)
	{
		numberFormat.setMinimumFractionDigits(n);
		numberFormat.setMaximumFractionDigits(n);
	}
	
	public static void setMaxNumFractionDigits(int n)
	{
		numberFormat.setMaximumFractionDigits(n);
	}
	
	public static NumberFormat getCurrentNumberFormat()
	{
		return numberFormat;
	}
	
	public static void setDefaultNumberFormat()
	{
		if (numberFormat != null)
		{
			numberFormat.setMaximumFractionDigits(0);
			numberFormat.setMinimumFractionDigits(0);
		}
	}
}
